@@ -63,6 +63,8 @@ from wlroots.wlr_types import (
     xdg_decoration_v1,
 )
 from wlroots.wlr_types.cursor import Cursor, WarpMode
+from wlroots.wlr_types.idle_inhibit_v1 import IdleInhibitorManagerV1, IdleInhibitorV1
+from wlroots.wlr_types.idle_notify_v1 import IdleNotifierV1
 from wlroots.wlr_types.keyboard import Keyboard
 from wlroots.wlr_types.layer_shell_v1 import LayerShellV1, LayerSurfaceV1
 from wlroots.wlr_types.output_management_v1 import (
@@ -81,6 +83,7 @@ from wlroots.wlr_types.scene import (
     SceneBuffer,
     SceneNode,
     SceneNodeType,
+    SceneRect,
     SceneSurface,
     SceneTree,
 )
@@ -98,6 +101,7 @@ from libqtile.backend.wayland.output import Output
 from libqtile.command.base import expose_command
 from libqtile.config import ScreenRect
 from libqtile.log_utils import logger
+from libqtile.utils import QtileError
 
 try:
     # Continue if ffi not built, so that docs can be built without wayland deps.
@@ -106,7 +110,8 @@ except ModuleNotFoundError:
     print("Warning: Wayland backend not built. Backend will not run.")
 
 if TYPE_CHECKING:
-    from typing import Any, Generator
+    from collections.abc import Generator
+    from typing import Any
 
     from cairocffi import ImageSurface
     from pywayland.server import Listener
@@ -312,7 +317,9 @@ class Core(base.Core, wlrq.HasListeners):
             SceneTree.create(self.windows_tree),  # Overlay
         ]
         self.mid_window_tree = self.layer_trees.pop(2)
-        self.wallpapers: dict[config.Screen, tuple[SceneBuffer, ImageSurface]] = {}
+        self.wallpapers: dict[
+            config.Screen, tuple[SceneBuffer | SceneRect, ImageSurface | None]
+        ] = {}
 
         # Add support for additional protocols
         ExportDmabufManagerV1(self.display)
@@ -326,6 +333,9 @@ class Core(base.Core, wlrq.HasListeners):
         self.add_listener(
             output_power_manager.set_mode_event, self._on_output_power_manager_set_mode
         )
+        self.idle = IdleNotifierV1(self.display)
+        idle_ihibitor_manager = IdleInhibitorManagerV1(self.display)
+        self.add_listener(idle_ihibitor_manager.new_inhibitor_event, self._on_new_idle_inhibitor)
         PrimarySelectionV1DeviceManager(self.display)
         virtual_keyboard_manager_v1 = vkeyboard.VirtualKeyboardManagerV1(self.display)
         self.add_listener(
@@ -390,6 +400,16 @@ class Core(base.Core, wlrq.HasListeners):
     def name(self) -> str:
         return "wayland"
 
+    @property
+    def active_keyboard(self) -> inputs.Keyboard | None:
+        keyboard = self.seat.get_keyboard()
+        if keyboard is not None:
+            for kb in self.keyboards:
+                if kb.keyboard._ptr == keyboard._ptr:
+                    return kb
+
+        return None
+
     def finalize(self) -> None:
         self.finalize_listeners()
         self._poll()
@@ -674,6 +694,7 @@ class Core(base.Core, wlrq.HasListeners):
 
     def _on_cursor_button(self, _listener: Listener, event: pointer.PointerButtonEvent) -> None:
         assert self.qtile is not None
+        self.idle.notify_activity(self.seat)
         found = None
         pressed = event.button_state == input_device.ButtonState.PRESSED
         if pressed:
@@ -712,6 +733,7 @@ class Core(base.Core, wlrq.HasListeners):
 
     def _on_cursor_motion(self, _listener: Listener, event: pointer.PointerMotionEvent) -> None:
         assert self.qtile is not None
+        self.idle.notify_activity(self.seat)
 
         dx = event.delta_x
         dy = event.delta_y
@@ -744,6 +766,7 @@ class Core(base.Core, wlrq.HasListeners):
         self, _listener: Listener, event: pointer.PointerMotionAbsoluteEvent
     ) -> None:
         assert self.qtile is not None
+        self.idle.notify_activity(self.seat)
 
         x, y = self.cursor.absolute_to_layout_coords(event.pointer.base, event.x, event.y)
         self.cursor.move(x - self.cursor.x, y - self.cursor.y)
@@ -757,6 +780,7 @@ class Core(base.Core, wlrq.HasListeners):
         _listener: Listener,
         event: pointer.PointerPinchBeginEvent,
     ) -> None:
+        self.idle.notify_activity(self.seat)
         self._gestures.send_pinch_begin(self.seat, event.time_msec, event.fingers)
 
     def _on_cursor_pinch_update(
@@ -773,6 +797,7 @@ class Core(base.Core, wlrq.HasListeners):
         _listener: Listener,
         event: pointer.PointerPinchEndEvent,
     ) -> None:
+        self.idle.notify_activity(self.seat)
         self._gestures.send_pinch_end(self.seat, event.time_msec, event.cancelled)
 
     def _on_cursor_swipe_begin(
@@ -780,6 +805,7 @@ class Core(base.Core, wlrq.HasListeners):
         _listener: Listener,
         event: pointer.PointerSwipeBeginEvent,
     ) -> None:
+        self.idle.notify_activity(self.seat)
         self._gestures.send_swipe_begin(self.seat, event.time_msec, event.fingers)
 
     def _on_cursor_swipe_update(
@@ -794,6 +820,7 @@ class Core(base.Core, wlrq.HasListeners):
         _listener: Listener,
         event: pointer.PointerSwipeEndEvent,
     ) -> None:
+        self.idle.notify_activity(self.seat)
         self._gestures.send_swipe_end(self.seat, event.time_msec, event.cancelled)
 
     def _on_cursor_hold_begin(
@@ -801,6 +828,7 @@ class Core(base.Core, wlrq.HasListeners):
         _listener: Listener,
         event: pointer.PointerHoldBeginEvent,
     ) -> None:
+        self.idle.notify_activity(self.seat)
         self._gestures.send_hold_begin(self.seat, event.time_msec, event.fingers)
 
     def _on_cursor_hold_end(
@@ -808,6 +836,7 @@ class Core(base.Core, wlrq.HasListeners):
         _listener: Listener,
         event: pointer.PointerHoldEndEvent,
     ) -> None:
+        self.idle.notify_activity(self.seat)
         self._gestures.send_hold_end(self.seat, event.time_msec, event.cancelled)
 
     def _on_new_pointer_constraint(
@@ -833,6 +862,20 @@ class Core(base.Core, wlrq.HasListeners):
         device = self._add_new_pointer(new_pointer_event.new_pointer.pointer.base)
         logger.info("New virtual pointer: %s %s", *device.get_info())
 
+    def _on_new_idle_inhibitor(
+        self, _listener: Listener, idle_inhibitor: IdleInhibitorV1
+    ) -> None:
+        logger.debug("Signal: idle_inhibitor new_inhibitor")
+
+        for win in self.qtile.windows_map.values():
+            if isinstance(win, window.Window | window.Static):
+                win.surface.for_each_surface(win.add_idle_inhibitor, idle_inhibitor)
+                if idle_inhibitor.data:
+                    # We break if the .data attribute was set, because that tells us
+                    # that `win.add_idle_inhibitor` identified this inhibitor as
+                    # belonging to that window.
+                    break
+
     def _on_input_inhibitor_activate(self, _listener: Listener, _data: Any) -> None:
         logger.debug("Signal: input_inhibitor activate")
         assert self.qtile is not None
@@ -1084,12 +1127,12 @@ class Core(base.Core, wlrq.HasListeners):
                     self.cursor.set_xcursor(self._cursor_manager, "default")
                     self.seat.pointer_notify_clear_focus()
 
-            if win is not self.qtile.current_window:
-                if self._hovered_window is not win:
-                    # We only want to fire client_mouse_enter once, so check
-                    # self._hovered_window.
-                    hook.fire("client_mouse_enter", win)
+            if self._hovered_window is not win:
+                # We only want to fire client_mouse_enter once, so check
+                # self._hovered_window.
+                hook.fire("client_mouse_enter", win)
 
+            if win is not self.qtile.current_window:
                 if motion is not None and self.qtile.config.follow_mouse_focus:
                     if isinstance(win, window.Static):
                         self.qtile.focus_screen(win.screen.index, False)
@@ -1440,20 +1483,47 @@ class Core(base.Core, wlrq.HasListeners):
         logger.warning("Couldn't determine what was under the pointer. Please report.")
         return None
 
+    def check_idle_inhibitor(self) -> None:
+        """
+        Checks if any window that is currently mapped has idle inhibitor
+        and if so inhibits idle
+        """
+        assert self.qtile is not None
+
+        for win in self.qtile.windows_map.values():
+            if not isinstance(win, window.Internal) and win.is_idle_inhibited:
+                # TODO: do we also need to check that the window is mapped?
+                self.idle.set_inhibited(True)
+                return
+
+        self.idle.set_inhibited(False)
+
     def get_screen_info(self) -> list[ScreenRect]:
         """Get the output information"""
         return [output.get_screen_info() for output in self.get_enabled_outputs()]
 
+    def _get_sym_from_code(self, keycode: int) -> str:
+        keyboard = self.active_keyboard
+        if keyboard is None:
+            raise QtileError("Unable to grab keycode. No active keyboard found.")
+        return keyboard._state.key_get_one_sym(keycode)
+
     def grab_key(self, key: config.Key | config.KeyChord) -> tuple[int, int]:
         """Configure the backend to grab the key event"""
-        keysym = xkb.keysym_from_name(key.key, case_insensitive=True)
+        if isinstance(key.key, str):
+            keysym = xkb.keysym_from_name(key.key, case_insensitive=True)
+        else:
+            keysym = self._get_sym_from_code(key.key)
         mask_key = wlrq.translate_masks(key.modifiers)
         self.grabbed_keys.append((keysym, mask_key))
         return keysym, mask_key
 
     def ungrab_key(self, key: config.Key | config.KeyChord) -> tuple[int, int]:
         """Release the given key event"""
-        keysym = xkb.keysym_from_name(key.key, case_insensitive=True)
+        if isinstance(key.key, str):
+            keysym = xkb.keysym_from_name(key.key, case_insensitive=True)
+        else:
+            keysym = self._get_sym_from_code(key.key)
         mask_key = wlrq.translate_masks(key.modifiers)
         self.grabbed_keys.remove((keysym, mask_key))
         return keysym, mask_key
