@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # Copyright (c) 2011-2012 Dustin Lacewell
 # Copyright (c) 2011 Mounier Florian
 # Copyright (c) 2012 Craig Barnes
@@ -14,6 +13,7 @@
 # Copyright (c) 2014 dequis
 # Copyright (c) 2014 Florian Scherf
 # Copyright (c) 2017 Dirk Hartmann
+# Copyright (c) 2024 Marco Paganini (auto_maximization code).
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -108,6 +108,21 @@ class MonadTall(_SimpleLayoutBase):
     methods. As mentioned the main pane is considered the top of the stack;
     moving up is counter-clockwise and moving down is clockwise.
 
+    ::
+
+        ---------------------          ---------------------
+        |            |      |          |            |      |
+        |            |______|          |            |Focus |
+        |            |      |          |            |      |
+        |            |______|          |            |______|
+        |            |      |          |            |______|
+        |            |      |          |            |      |
+        ---------------------          ---------------------
+
+    Setting ``auto_maximize`` will cause the focused secondary pane to be
+    automatically maximized on focus. The non-maximized panes will shrink to
+    the height specified by ``min_secondary_size``.
+
     The opposite is true if the layout is "flipped".
 
     ::
@@ -147,8 +162,10 @@ class MonadTall(_SimpleLayoutBase):
         Key([modkey, "shift"], "k", lazy.layout.shuffle_up()),
         Key([modkey], "i", lazy.layout.grow()),
         Key([modkey], "m", lazy.layout.shrink()),
-        Key([modkey], "n", lazy.layout.normalize()),
+        Key([modkey], "n", lazy.layout.reset()),
+        Key([modkey, "shift"], "n", lazy.layout.normalize()),
         Key([modkey], "o", lazy.layout.maximize()),
+        Key([modkey, "shift"], "s", lazy.layout.toggle_auto_maximize()),
         Key([modkey, "shift"], "space", lazy.layout.flip()),
     """
 
@@ -156,6 +173,7 @@ class MonadTall(_SimpleLayoutBase):
     _right = 1
 
     defaults = [
+        ("auto_maximize", False, "Maximize secondary windows on focus."),
         ("border_focus", "#ff0000", "Border colour(s) for the focused window."),
         ("border_normal", "#000000", "Border colour(s) for un-focused windows."),
         ("border_width", 2, "Border width."),
@@ -242,10 +260,26 @@ class MonadTall(_SimpleLayoutBase):
         self.clients.add_client(client, client_position=self.new_client_position)
         self.do_normalize = True
 
+    def focus(self, client):
+        super().focus(client)
+        # Only maximize the window in the secondary pane when focus is *not* in
+        # the main pane. Doing so in the main pane causes the last secondary
+        # window to always be in focus when switching from secondary -> main.
+        if self.focused != 0:
+            self._maximize_focused_secondary()
+
     def remove(self, client: Window) -> Window | None:
         "Remove client from layout"
+        p = super().remove(client)
         self.do_normalize = True
-        return self.clients.remove(client)
+        # When auto_maximize is set and the user closes the first (topmost)
+        # secondary window, focus goes back to the main window. In this case,
+        # we WANT to force redraw of the windows in the secondary pane so we
+        # get a maximized topmost window again.
+        if self.auto_maximize and self.focused == 0 and len(self.clients) > 2:
+            # This will also trigger secondary maximization, if needed.
+            self.focus(self.clients[1])
+        return p
 
     @expose_command()
     def set_ratio(self, ratio):
@@ -274,6 +308,14 @@ class MonadTall(_SimpleLayoutBase):
             self.align = self._left
         self.normalize(redraw)
 
+    @expose_command()
+    def toggle_auto_maximize(self):
+        "Toggle auto maximize secondary window on focus."
+        self.auto_maximize = not self.auto_maximize
+        self.normalize(True)
+        if self.focused != 0:
+            self._maximize_focused_secondary()
+
     def _maximize_main(self):
         "Toggle the main pane between min and max size"
         if self.ratio <= 0.5 * (self.max_ratio + self.min_ratio):
@@ -304,6 +346,46 @@ class MonadTall(_SimpleLayoutBase):
         else:
             self._grow_secondary(maxed_size)
 
+    def _maximize_focused_secondary(self):
+        "Maximize the 'non-maximized' focused secondary pane"
+
+        # Return immediately if no self.group.screen
+        # (this may happen when moving windows across screens.)
+        if self.group.screen is None:
+            return
+
+        # If auto_maximize is off, return immediately.
+        if not self.auto_maximize:
+            return
+
+        # if we have 1 or 2 panes, do nothing.
+        if len(self.clients) < 3:
+            return
+
+        # Recalculate relative_sizes
+        self.normalize(redraw=False)
+        if len(self.relative_sizes) == 0:
+            return
+
+        # If the focused window (self.focused) is 0 (main pane), adjust
+        # focused to work directly on the secondary pane windows.
+        focused = max(1, self.focused)
+
+        n = len(self.clients) - 2  # total shrinking clients
+        # total size of collapsed secondaries
+        collapsed_size = self.min_secondary_size * n
+        nidx = max(0, focused - 1)  # focused size index
+        # total height of maximized secondary
+        maxed_size = self.group.screen.dheight - collapsed_size
+
+        # Maximize if window is not already maximized.
+        if (
+            abs(self._get_absolute_size_from_relative(self.relative_sizes[nidx]) - maxed_size)
+            >= self.change_size
+        ):
+            self._grow_secondary(maxed_size)
+            self.group.layout_all()
+
     @expose_command()
     def maximize(self):
         "Grow the currently focused client to the max size"
@@ -461,7 +543,7 @@ class MonadTall(_SimpleLayoutBase):
         """
         left = amt  # track unused shrink amount
         # for each client before specified index
-        for idx in range(0, cidx):
+        for idx in range(cidx):
             # shrink by whatever is left-over of original amount
             left -= left - self._shrink(idx, left)
         # return unused shrink amount
@@ -481,7 +563,7 @@ class MonadTall(_SimpleLayoutBase):
         per_amt = amt / cidx
         left = amt  # track unused shrink amount
         # for each client before specified index
-        for idx in range(0, cidx):
+        for idx in range(cidx):
             # shrink by equal amount and track left-over
             left -= per_amt - self._shrink(idx, per_amt)
         # apply non-equal shrinkage secondary pass
@@ -615,7 +697,7 @@ class MonadTall(_SimpleLayoutBase):
         """
         # split grow amount among number of clients
         per_amt = amt / cidx
-        for idx in range(0, cidx):
+        for idx in range(cidx):
             self._grow(idx, per_amt)
 
     def grow_down_shared(self, cidx, amt):
@@ -878,7 +960,8 @@ class MonadWide(MonadTall):
         Key([modkey, "shift"], "k", lazy.layout.shuffle_up()),
         Key([modkey], "i", lazy.layout.grow()),
         Key([modkey], "m", lazy.layout.shrink()),
-        Key([modkey], "n", lazy.layout.normalize()),
+        Key([modkey], "n", lazy.layout.reset()),
+        Key([modkey, "shift"], "n", lazy.layout.normalize()),
         Key([modkey], "o", lazy.layout.maximize()),
         Key([modkey, "shift"], "space", lazy.layout.flip()),
     """
