@@ -17,13 +17,12 @@
 # LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 # SOFTWARE.
+from collections.abc import Callable
 from contextlib import suppress
 from functools import partial
 from pathlib import Path
 
 # dbus_next is incompatible with deferred type evaluation
-from typing import Callable, List, Optional
-
 import cairocffi
 from dbus_next import InterfaceNotFoundError, InvalidBusNameError, InvalidObjectPathError
 from dbus_next.aio import MessageBus
@@ -42,6 +41,8 @@ from libqtile.images import Img
 from libqtile.log_utils import logger
 from libqtile.utils import add_signal_receiver, create_task
 
+ICON_FORMATS = [".png", ".svg"]
+
 # StatusNotifier seems to have two potential interface names.
 # While KDE appears to be the default, we should also listen
 # for items on freedesktop.
@@ -237,21 +238,26 @@ class StatusNotifierItem:  # noqa: E303
         # need to catch an error if we can't read it.
         # We can't use hasattr to check this as the method will be created
         # where we've used the default XML spec to provide the object introspection
+        icon_name = ""
         try:
             icon_name = await self.item.get_icon_name()
         except DBusError:
             return
 
-        try:
-            icon_path = await self.item.get_icon_theme_path()
-        except (AttributeError, DBusError):
-            icon_path = None
+        # We only need to do these searches if there's an icon name provided by
+        # the app. We also don't want to do the recursive lookup with an empty
+        # icon name as, otherwise, the glob will match things that are not images.
+        if icon_name:
+            try:
+                icon_path = await self.item.get_icon_theme_path()
+            except (AttributeError, DBusError):
+                icon_path = None
 
-        if icon_path:
-            self.icon = self._get_custom_icon(icon_name, Path(icon_path))
+            if icon_path:
+                self.icon = self._get_custom_icon(icon_name, Path(icon_path))
 
-        if not self.icon:
-            self.icon = self._get_xdg_icon(icon_name)
+            if not self.icon:
+                self.icon = self._get_xdg_icon(icon_name)
 
         if not self.icon and fallback:
             # Use fallback icon libqtile/resources/status_notifier/fallback_icon.png
@@ -278,7 +284,7 @@ class StatusNotifierItem:  # noqa: E303
 
     def _get_custom_icon(self, icon_name, icon_path):
         icon = None
-        for ext in [".png", ".svg"]:
+        for ext in ICON_FORMATS:
             path = icon_path / f"{icon_name}{ext}"
             if path.is_file():
                 icon = path
@@ -287,7 +293,9 @@ class StatusNotifierItem:  # noqa: E303
         else:
             # No icon found at the image path, let's search recursively
             glob = icon_path.rglob(f"{icon_name}.*")
-            found = [icon for icon in glob if icon.is_file()]
+            found = [
+                icon for icon in glob if icon.is_file() and icon.suffix.lower() in ICON_FORMATS
+            ]
 
             # Found a matching icon in subfolder
             if found:
@@ -461,13 +469,13 @@ class StatusNotifierWatcher(ServiceInterface):  # noqa: E303
 
     def __init__(self, service: str):
         super().__init__(service)
-        self._items: List[str] = []
-        self._hosts: List[str] = []
+        self._items: list[str] = []
+        self._hosts: list[str] = []
         self.service = service
-        self.on_item_added: Optional[Callable] = None
-        self.on_host_added: Optional[Callable] = None
-        self.on_item_removed: Optional[Callable] = None
-        self.on_host_removed: Optional[Callable] = None
+        self.on_item_added: Callable | None = None
+        self.on_host_added: Callable | None = None
+        self.on_item_removed: Callable | None = None
+        self.on_host_removed: Callable | None = None
 
     async def start(self):
         # Set up and register the service on ths bus
@@ -597,20 +605,20 @@ class StatusNotifierHost:  # noqa: E303
     """
 
     def __init__(self):
-        self.watchers: List[StatusNotifierWatcher] = []
-        self.items: List[StatusNotifierItem] = []
+        self.watchers: list[StatusNotifierWatcher] = []
+        self.items: list[StatusNotifierItem] = []
         self.name = "qtile"
         self.icon_theme: str = None
         self.started = False
-        self._on_item_added: List[Callable] = []
-        self._on_item_removed: List[Callable] = []
-        self._on_icon_changed: List[Callable] = []
+        self._on_item_added: list[Callable] = []
+        self._on_item_removed: list[Callable] = []
+        self._on_icon_changed: list[Callable] = []
 
     async def start(
         self,
-        on_item_added: Optional[Callable] = None,
-        on_item_removed: Optional[Callable] = None,
-        on_icon_changed: Optional[Callable] = None,
+        on_item_added: Callable | None = None,
+        on_item_removed: Callable | None = None,
+        on_icon_changed: Callable | None = None,
     ):
         """
         Starts the host if not already started.
